'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.default = configure;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _vm = require('vm');

var _vm2 = _interopRequireDefault(_vm);

var _crypto = require('crypto');

var _resolve = require('resolve');

var _resolve2 = _interopRequireDefault(_resolve);

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _babelCore = require('babel-core');

var _mkdirp = require('mkdirp');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var called = false;
function configure(entrypoint, overrideRequires, opts, onValue, onError) {
  if (called) throw new Error('You can only use babel-live once per project');
  called = true;

  opts = opts || {};
  var babelCache = opts.babelCache ? _path2.default.resolve(opts.babelCache) : null;
  if (babelCache) {
    delete opts.babelCache;
    (0, _mkdirp.sync)(babelCache);
  }
  var fallbackRequire = opts.fallbackRequire;
  if (fallbackRequire) {
    delete opts.fallbackRequire;
  }

  function handleError(ex) {
    var err = '';
    if (ex.codeFrame) {
      err = ex.message + '\n' + ex.codeFrame;
    } else {
      err = ex.stack;
    }
    if (onError) {
      onError(err);
    } else if (fallbackRequire) {
      console.error(err);
    } else {
      throw err;
    }
    if (fallbackRequire) {
      var result = babelRequire(fallbackRequire);
      onValue(result);
      return result;
    }
  }

  if (opts.sourceMap !== false) opts.sourceMap = "inline";

  var requireInProgress = false;
  var extraRequireNeeded = false;
  var requireCache = {};
  // filename => fn(module, exports, require, __filename, __dirname)
  var moduleCache = {};
  var maps = {};
  /*
  // TODO: source maps in a VM are different
  sourceMapSupport.install({
    handleUncaughtExceptions: false,
    environment: 'node',
    retrieveSourceMap(source) {
      const map = maps[source];
      if (map) {
        return {
          url: null,
          map,
        };
      } else {
        return null;
      }
    },
  });
  */

  function invalidate(filename) {
    console.log('detected file change: ' + filename);
    requireCache = {};
    moduleCache[filename] = null;
    doRequire();
  }
  function doRequire() {
    if (!requireInProgress) {
      requireInProgress = true;
      try {
        var result = babelRequire(entrypoint);
        onValue(result);
      } catch (ex) {
        handleError(ex);
      }
      setTimeout(function () {
        requireInProgress = false;
        if (extraRequireNeeded) {
          extraRequireNeeded = false;
          doRequire();
        }
      }, 2000);
    } else {
      extraRequireNeeded = true;
    }
  }

  var watching = {};
  function watch(filename) {
    if (watching[filename]) return;
    watching[filename] = true;
    var w = _chokidar2.default.watch(filename, {
      persistent: true,
      usePolling: true,
      interval: 100
    });
    w.on('error', function (err) {
      console.log('error watching file');
      console.log('probably nothing to worry about');
      console.log(err.message);
    });
    w.on('change', function () {
      return invalidate(filename);
    });
  }

  function babelRequire(filename) {
    filename = _path2.default.resolve(filename);
    if (!/\.js$/.test(filename)) {
      return require(filename);
    }
    if (!/node_modules/.test(filename)) {
      watch(filename);
    }
    if (requireCache[filename]) return requireCache[filename].exports;
    var fn = moduleCache[filename];
    if (!fn) {
      var src = !/node_modules/.test(filename) ? babelLoad(filename) : _fs2.default.readFileSync(filename, 'utf8');
      fn = _vm2.default.runInThisContext('(function(module,exports,require,__filename,__dirname){' + src + '\n})', filename);
      moduleCache[filename] = fn;
    }
    requireCache[filename] = {
      exports: {}
    };
    function proxiedRequire(id) {
      if (overrideRequires[id]) {
        return overrideRequires[id];
      }

      if (_resolve2.default.isCore(id)) {
        return require(id);
      }

      var p = _resolve2.default.sync(id, {
        basedir: _path2.default.dirname(filename),
        extensions: ['.js', '.json', '.node']
      });

      return babelRequire(p);
    }
    proxiedRequire.resolve = function (id) {
      return _resolve2.default.sync(id, {
        basedir: _path2.default.dirname(filename),
        extensions: ['.js', '.json', '.node']
      });
    };
    var sandbox = {
      'module': requireCache[filename],
      'exports': requireCache[filename].exports,
      'require': proxiedRequire,
      '__filename': filename,
      '__dirname': _path2.default.dirname(filename)
    };
    fn(sandbox.module, sandbox.exports, sandbox.require, sandbox.__filename, sandbox.__dirname);
    return requireCache[filename].exports;
  }

  function babelLoad(filename) {
    var src = _fs2.default.readFileSync(filename, 'utf8');
    opts.filename = filename;
    var hash = void 0,
        result = void 0;
    if (babelCache) {
      hash = (0, _crypto.createHash)('sha1').update(src).digest('hex');
      try {
        result = JSON.parse(_fs2.default.readFileSync(_path2.default.join(babelCache, hash + '.json'), 'utf8'));
      } catch (ex) {
        if (ex.code !== 'ENOENT') {
          throw ex;
        }
      }
    }
    if (!result) {
      result = (0, _babelCore.transform)(src, (0, _extends3.default)({}, opts, { sourceMaps: 'both', ast: false }));
      if (babelCache) {
        _fs2.default.writeFileSync(_path2.default.join(babelCache, hash + '.json'), (0, _stringify2.default)(result));
      }
    }
    maps[filename] = result.map;
    return result.code;
  }
  try {
    var result = babelRequire(entrypoint);
    onValue(result);
    return {
      reRun: function reRun() {
        requireCache = {};
        doRequire();
      }
    };
  } catch (ex) {
    return handleError(ex);
  }
}